<?

/*
 			phpCached : cache PHP
			NOTE : il est possible de monter des partitions en RAM ainsi on utilise
			un cache RAM, beaucoup plus rapide que l'utilisation sur disk.

 			Version: 0.1 Beta 		
 			
 */

# This file is part of phpCached.
# Copyright (c) 2005 Stephane Depierrepont (toorop@cyborgnation.org). 
# All rights reserved.
#
# phpCached is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# phpCached is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with phpCached; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#



class phpCached{	

	protected  $cache_size=200000; 						# Taille du cache en octets. 
	protected  $cache_path_root=false;					# Repertoire de base du cache.
	protected  $cache_path=false;						# Repertoire ou se trouve le fichier cache
	protected  $cache_file=false;						# Fichier cache
	protected  $default_cached_time=1860;				# Duree de mise en cache par defaut (sec)
	protected  $start_cleaning_for_key_older_than=1860; # Lors du nettoyage, premier seuil de suppression (sec)
	
	
	/* Constructeur */
	function __construct($cache_path_root=false){
		if ($cache_path_root==false)throw new Exception('$cache_path_root not set');		
		/* On verifie que le repertoire de cache est OK*/		
		$this->_verifCacheDir($cache_path_root);
		return true;					
	}

	/*
	**************************
	* Methodes protegees
	**************************	
	*/	
	
	/**
	 * Retourne le hash utilise comme identifiant
	 *
	 * @param string $r
	 * @return string
	 */
	protected function getHash($r){
		return sha1($r)	;
	}	
	
	/**
	 * Supprime l'enregistrement correspondant a la clef du cache
	 *
	 * @param string $key
	 * @return bool true
	 */
	public function	delFromCache($key){
		if(!file_exists($this->cache_dir.'/'.substr($key,0,2).'/'.$key))throw new Exception($key.' is not a existing key.');
		unlink($this->cache_dir.'/'.substr($key,0,2).'/'.$key);
		return true;
	}	
	
	/**
	 * retourne la valeur en cache ou false si elle ne s'y trouve pas
	 *
	 * @param string $key : La clef correspondant a l'enregistrement
	 * @param int $cached_time : Validite de la valeur cachee en seconde
	 * @return bool false si il n'y a pas de valeur associee a la clef, sinon string value
	 */
	protected function getFromCache($key=false,$cached_time=false){	
		if ($key==false)throw new Exception('$key var not set in phpCached->getFromCache($key).');
		if($cached_time==false)$cached_time=$this->default_cached_time;
		$value=false;
		/* On recupere le path et le fichier cache */
		$this->_keyToFilePath($key);
		/* On regarde si clef est cachee et si elle est toujours valide */
		if (file_exists($this->cache_file)){			
			clearstatcache();
			if (filemtime($this->cache_file) + $cached_time < time()){	
				@unlink($this->cache_file);
				return false; # Le cache a expire
			} else {
				$f = fopen($this->cache_file, 'r');
				if ($f === false)throw new Exception('Unable to open cache file '.$this->cache_file.' in phpCached->getFromCache');
				if(false===$value=fread($f,filesize($this->cache_file)))throw new Exception('Unable to read cache file '.$this->cache_file.' in phpCached->getFromCache');
				@fclose($f);
				$value=unserialize($value);	
				return $value;			
			}
		} 
		return false;			
	}

	/**
	 * Ajoute un couple key -> value en cache
	 *
	 * @param string $key
	 * @param unknown_type $value
	 * @return bool true
	 */
	protected function putInCache($key,$value){
		/* Nettoyage du cache si necessaire */		
		if($this->_getCacheSize()>=$this->cache_size)$this->_cleanCache();	
		/* On serialise $value */
		$value=serialize($value);
		if (!is_dir($this->cache_path))	mkdir($this->cache_path,0700,true);	
		$f = @fopen($this->cache_file, 'w');	
		if ($f===false)throw new Exception('Unable to open cache file in phpCached->putIncache');
		if(false===fwrite($f,$value))throw new Exception('Unable to read cache file '.$this->cache_file);
		@fclose($f);
		return true;		
	}	
	
	/*
	**************************
	* Methodes privees
	**************************	
	*/

	/**
	 * Verifie que le repertoire de cache existe et que l'on peut y ecrire,
	 * essaye de le creer si non.
	 *
	 * @param string $d : repertoire de cache
	 * @return bool true
	 */
	
	private function _verifCacheDir($d){
		$this->cache_dir=$d.'/phpCached';	
		if (is_dir($this->cache_dir)&& is_writable($this->cache_dir))return true;
		if (!is_dir($this->cache_dir) and is_dir($d) and is_writable($d)){
			mkdir($this->cache_dir);
			return true; 
		}else throw new Exception('cache_dir: '.$d.' Not found or not writable');
	}
	
	/**
	 * Retourne la taille occupee par le cache
	 *
	 * @return int : taille du cache en octets
	 */
	private function _getCacheSize($dir=false) {
		static $size;
		if($dir===false){$dir=$this->cache_dir;$size=0;}		
		$dp=opendir($dir);
		do {
			$itm=readdir($dp);	
			if (is_dir("$dir/$itm") && ($itm!=".") && ($itm!="..") && ($itm!=""))$this->_getCacheSize("$dir/$itm");
        	elseif (($itm!=".") && ($itm!="..") && ($itm!="")) $size = $size + filesize($dir.'/'.$itm);
		} while ($itm != false);
		@closedir($dp);
		return $size;
	}
	
	/**
	 * Nettoie le cache si sa taille maxi est atteinte
	 *
	 * @return bool true
	 */
	private function _cleanCache($dir=false){
		if($dir===false)$dir=$this->cache_dir;
		$now=time();
		$dp=opendir($dir);
		do {
			$itm=readdir($dp);	
			if (($itm!=".") && ($itm!="..") && ($itm!="")){		
				if (is_dir($dir.'/'.$itm)) {
					/* Si le rep est vide on le supprime */
					if (true===$this->_isEmptyDir($dir.'/'.$itm))rmdir($dir.'/'.$itm);
					else $this->_cleanCache($dir.'/'.$itm);
				}
				elseif($now - filemtime($dir.'/'.$itm) > $this->start_cleaning_for_key_older_than)unlink($dir.'/'.$itm);
			}
		} while ($itm != false);
		#echo 'La taille occupee par le cache apres nettoyage est :'.$this->_getCacheSize().' Octets.<br/>';
		closedir($dp);
		if ($this->_getCacheSize($this->cache_dir)>=$this->cache_size){
			$this->start_cleaning_for_key_older_than=$this->start_cleaning_for_key_older_than/2;	
			$this->_cleanCache();
		}
		return true;
	}
	
	/**
	 * Calcul le chemin vers la clef. Il est important de splitter le repertoire de base
	 * car en fonction des systeme de fichier, le nombre de fichiers dans un meme repertoire
	 * est limit a un certain nombre.
	 *
	 * @param string $key
	 * @return bool true
	 */
	private function _keyToFilePath($key=false){
		if ($key===false) throw new Exception('$Key not set in private _keyToFilePath()');
		if (strlen($key)<4) {
			$this->cache_path=$this->cache_dir;			
		} else {
			$this->cache_path=$this->cache_dir.'/'.substr($key,0,2);
		}
		$this->cache_file=$this->cache_path.'/'.$key;
		return true;		
	}
	
	/**
	 * Verifie si un repertoire est vide
	 *
	 * @param string $dir: repertoire a tester
	 * @return bool
	 */
	private function _isEmptyDir($dir=false){
		if ($dir===false) throw new Exception('$dir not set in private _isEmptyDir()');
		$result=true;
		$dp=opendir($dir);
		do {
			$itm=readdir($dp);
			if(($itm!=".") && ($itm!="..") && ($itm!="")){$result=false;break;}
		} while ($itm != false);
		@closedir($dp);
		return $result;			
	}
}
?>